import os as __os
from lumipy.client import Client as __Client
import lusid.api as __apis
from lusid.utilities import ApiClientFactory as __ApiClientFactory
from lusidjam import RefreshingToken as __RefreshingToken

def __camel_to_snake(name):
    import re
    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()



__secrets_path = __os.getenv("FBN_SECRETS_PATH")

__api_factory = __ApiClientFactory(
    token=__RefreshingToken(),
    api_secrets_filename=__secrets_path,
    app_name="LusidJupyterNotebook",
)
__api_exports = []

def __create_api_variables(api_factory, module):
    """
    Create and export variables for each API class in the given module.
    The variables will be named in the format <name>_api where <name> is the lowercase class name.
    Adds each variable name to the __all__ list for module exports.
    """
    global __api_exports
    for attr_name in dir(module):
        attr = getattr(module, attr_name)
        if isinstance(attr, type) and attr_name.endswith('Api'):
            variable_name = __camel_to_snake(attr_name[:-3]) + '_api'
            globals()[variable_name] = api_factory.build(attr)
            __api_exports.append(variable_name)


__create_api_variables(__api_factory, __apis)


__api_exports.append(__Client(api_secrets_filename=__os.environ.get('FBN_SECRETS_PATH',None)))
__all__ = __api_exports


from IPython.display import Markdown as render_markdown
import os as __os
with open(__os.path.join(__os.path.dirname(__file__), 'PRELOADED_VARS.md'), 'r') as __f:
    PRELOADED_VARS_MARKDOWN = __f.read()
    
def display_preloaded_var_markdown():
    return render_markdown(PRELOADED_VARS_MARKDOWN)


import argparse
import yaml
import os
import shutil

def parse_args():
    
    parser = argparse.ArgumentParser(description="Configure lusid_express settings.")
    parser.add_argument('-e','--enable', nargs='+', type=str, choices=['vars', 'magic'], help='Enable feature(s).')
    parser.add_argument('-d','--disable', nargs='+', type=str, choices=['vars', 'magic'], help='Disable feature(s).')
    return parser.parse_args()

def update_config(args):
    config_path = os.path.join(os.path.dirname(__file__), 'config.yaml')
    if os.path.exists(config_path):
        with open(config_path, 'r') as f:
            config = yaml.safe_load(f) or {}
    else:
        config = {'features': []}

    enabled_features = set(config.get('features', []))

    if args.enable:
        enabled_features.update(args.enable)

    if args.disable:
        enabled_features.difference_update(args.disable)

    config['features'] = list(enabled_features)

    with open(config_path, 'w') as f:
        yaml.safe_dump(config, f)


def copy_startup_file():
    ipython_startup_dir = os.path.expanduser('~/.ipython/profile_default/startup/')
    target_file = os.path.join(ipython_startup_dir, '00-load_lusid_express.py')
    source_file = os.path.join(os.path.dirname(__file__), 'load.le')

    # Ensure the IPython startup directory exists
    os.makedirs(ipython_startup_dir, exist_ok=True)

    # Copy the load.py file if it does not already exist
    if not os.path.exists(target_file):
        shutil.copy(source_file, target_file)
        print(f"File {source_file} copied to {target_file}")
    else:
        print(f"File {target_file} already exists. No action taken.")
        
        
        
def main():
    args = parse_args()
    update_config(args)
    copy_startup_file()
    print("Configuration updated successfully! Changes will be applied after kernel restart.")

if __name__ == "__main__":
    main()
